@@ -6,13 +6,13 @@ import sentry_sdk
 from rich.console import Console
 from rich.syntax import Syntax
 
-from piperider_cli import __version__, sentry_dns, sentry_env, event
 import piperider_cli.dbtutil as dbtutil
+from piperider_cli import __version__, sentry_dns, sentry_env, event
 from piperider_cli.assertion_generator import AssertionGenerator
 from piperider_cli.cloud_connector import CloudConnector
 from piperider_cli.compare_report import CompareReport
 from piperider_cli.event import UserProfileConfigurator
-from piperider_cli.event.track import TrackCommand, BetaGroup
+from piperider_cli.event.track import TrackCommand
 from piperider_cli.exitcode import EC_ERR_TEST_FAILED
 from piperider_cli.feedback import Feedback
 from piperider_cli.generate_report import GenerateReport
@@ -154,6 +154,7 @@ def diagnose(**kwargs):
 @click.option('--output', '-o', default=None, type=click.STRING, help='Directory to save the results.')
 @click.option('--skip-report', is_flag=True, help='Skip generating report.')
 @click.option('--dbt-state', default=None, help='Directory of the the dbt state.')
+@click.option('--dbt-run-results', is_flag=True, help='Associate with dbt run results')
 @click.option('--report-dir', default=None, type=click.STRING, help='Use a different report directory.')
 @click.option('--upload', is_flag=True, help='Upload the report to the PipeRider Cloud.')
 @click.option('--open', is_flag=True, help='Opens the generated report in the system\'s default browser')
@@ -167,19 +168,38 @@ def run(**kwargs):
     open_report = kwargs.get('open')
     skip_report = kwargs.get('skip_report')
     dbt_state_dir = kwargs.get('dbt_state')
+    dbt_run_results = kwargs.get('dbt_run_results')
     force_upload = kwargs.get('upload')
+
+    dbt_resources = None
+    if not sys.stdin.isatty():
+        metrics = []
+        models = []
+        for dbt_resource in sys.stdin:
+            if dbt_resource.startswith('source:'):
+                continue
+            elif dbt_resource.startswith('metric:'):
+                metrics.append(dbt_resource.rstrip().replace('metric:', 'metric.'))
+            else:
+                models.append(dbt_resource.rstrip())
+        dbt_resources = dict(metrics=metrics, models=models)
+
     ret = Runner.exec(datasource=datasource,
                       table=table,
                       output=output,
                       skip_report=skip_report,
                       dbt_state_dir=dbt_state_dir,
+                      dbt_run_results=dbt_run_results,
+                      dbt_resources=dbt_resources,
                       report_dir=kwargs.get('report_dir'))
     if ret in (0, EC_ERR_TEST_FAILED):
         auto_upload = CloudConnector.is_auto_upload()
         is_cloud_view = (force_upload or auto_upload)
 
         if CloudConnector.is_login() and is_cloud_view:
-            CloudConnector.upload_latest_report(report_dir=kwargs.get('report_dir'), debug=kwargs.get('debug'), open_report=open_report, force_upload=force_upload, auto_upload=auto_upload)
+            CloudConnector.upload_latest_report(report_dir=kwargs.get('report_dir'), debug=kwargs.get('debug'),
+                                                open_report=open_report, force_upload=force_upload,
+                                                auto_upload=auto_upload)
 
         if not skip_report:
             GenerateReport.exec(None, kwargs.get('report_dir'), output, open_report, is_cloud_view)
@@ -304,6 +324,8 @@ def cloud(**kwargs):
 @click.option('--report-dir', default=None, type=click.STRING, help='Use a different report directory.')
 @click.option('--datasource', default=None, type=click.STRING, metavar='DATASOURCE_NAME',
               help='Specify the datasource.')
+@click.option('--project', default=None, type=click.STRING, metavar='PROJECT_NAME',
+              help='Specify the project to upload.')
 @add_options(debug_option)
 def upload_report(**kwargs):
     """
@@ -312,20 +334,24 @@ def upload_report(**kwargs):
     report_path = kwargs.get('run')
     datasource = kwargs.get('datasource')
     report_dir = kwargs.get('report_dir')
+    project_name = kwargs.get('project')
     ret = CloudConnector.upload_report(report_path=report_path, datasource=datasource, report_dir=report_dir,
+                                       project_name=project_name,
                                        debug=kwargs.get('debug', False))
     return ret
 
 
 @cloud.command(name='compare-reports', short_help='Generate comparison report on PipeRider Cloud.', cls=TrackCommand)
-@click.option('--base', default=None, required=True, type=click.STRING,
+@click.option('--base', default=None, required=False, type=click.STRING,
               help='Specify the base report id or data source name. e.g., 123 or datasource:<name>')
-@click.option('--target', default=None, required=True, type=click.STRING,
+@click.option('--target', default=None, required=False, type=click.STRING,
               help='Specify the target report id or data source name. e.g., 123 or datasource:<name>')
 @click.option('--tables-from', default='all',
               type=click.Choice(['all', 'target-only', 'base-only'], case_sensitive=False),
               help='Show table comparison from base or target.')
 @click.option('--summary-file', default=None, type=click.STRING, help='Download the comparison summary markdown file.')
+@click.option('--project', default=None, type=click.STRING, metavar='PROJECT_NAME',
+              help='Specify the project to upload.')
 @add_options(debug_option)
 def cloud_compare_reports(**kwargs):
     """
@@ -335,9 +361,10 @@ def cloud_compare_reports(**kwargs):
     target = kwargs.get('target')
     tables_from = kwargs.get('tables_from')
     summary_file = kwargs.get('summary_file')
+    project_name = kwargs.get('project')
 
     ret = CloudConnector.compare_reports(base=base, target=target, tables_from=tables_from, summary_file=summary_file,
-                                         debug=kwargs.get('debug', False))
+                                         project_name=project_name, debug=kwargs.get('debug', False))
 
     if ret != 0:
         sys.exit(ret)
@@ -368,3 +395,20 @@ def login(**kwargs):
 def logout(**kwargs):
     ret = CloudConnector.logout()
     return ret
+
+
+@cloud.command(short_help='List projects on PipeRider Cloud.', cls=TrackCommand)
+@add_options(debug_option)
+def list_projects(**kwargs):
+    ret = CloudConnector.list_projects()
+    return ret
+
+
+@cloud.command(short_help='Select a project on PipeRider Cloud as default project.', cls=TrackCommand)
+@click.option('--project', default=None, type=click.STRING, metavar='PROJECT_NAME',
+              help='Specify the project name.')
+@add_options(debug_option)
+def select_project(**kwargs):
+    project_name = kwargs.get('project')
+    ret = CloudConnector.select_project(project_name=project_name)
+    return ret
