@@ -10,6 +10,7 @@ import readchar
 from rich.console import Console
 from rich.prompt import Prompt
 from sqlalchemy import create_engine, select, text
+from sqlalchemy.pool import SingletonThreadPool
 
 import piperider_cli.hack.datasource_inquirer_prompt as datasource_prompt
 from piperider_cli.error import PipeRiderTableConnectionError
@@ -85,9 +86,7 @@ class DataSource(metaclass=ABCMeta):
     def verify_connection(self):
         engine = self.get_engine_by_database()
         with engine.connect() as conn:
-            stmt = select(
-                text('1 as id')
-            )
+            stmt = select([text('1 as id')])
             conn.execute(stmt)
         return
 
@@ -105,6 +104,15 @@ class DataSource(metaclass=ABCMeta):
 
         return engine
 
+    @property
+    def threads(self):
+        if self.credential.get('threads'):
+            return self.credential.get('threads')
+        elif not isinstance(self.get_engine_by_database().pool, SingletonThreadPool):
+            return 5
+        else:
+            return 1
+
     def engine_args(self):
         return dict()
 
@@ -221,6 +229,32 @@ class DataSource(metaclass=ABCMeta):
         cls = answers['type']
         return cls, name
 
+    @abstractmethod
+    def _get_display_description(self):
+        """
+        get the description of the datasource to display.
+
+        Returns description
+
+        :return: str
+        """
+        raise NotImplementedError
+
+    def get_display_description(self):
+        # if dbt
+        dbt = self.args.get('dbt')
+        if dbt:
+            return f"profile={dbt.get('profile')}, target={dbt.get('target')}"
+        return self._get_display_description()
+
+    @abstractmethod
+    def get_database(self):
+        raise NotImplementedError
+
+    @abstractmethod
+    def get_schema(self):
+        raise NotImplementedError
+
 
 def _list_datasource_providers():
     from .snowflake import SnowflakeDataSource
