@@ -1,11 +1,11 @@
 import json
 import math
 import os
-import re
 import shutil
 import sys
 import uuid
 from datetime import datetime
+from typing import List, Optional
 
 from rich import box
 from rich.color import Color
@@ -24,6 +24,7 @@ from piperider_cli import event
 from piperider_cli.assertion_engine import AssertionEngine
 from piperider_cli.assertion_engine.recommender import RECOMMENDED_ASSERTION_TAG
 from piperider_cli.configuration import Configuration
+from piperider_cli.datasource import DataSource
 from piperider_cli.exitcode import EC_ERR_TEST_FAILED
 from piperider_cli.filesystem import FileSystem
 from piperider_cli.metrics_engine import MetricEngine, MetricEventHandler
@@ -67,7 +68,8 @@ class RichProfilerEventHandler(ProfilerEventHandler):
         self.table_completed = 0
 
     def _get_width(self, tables):
-        return max([len(x) for x in tables]), len(str(len(tables))) * 2 + 2
+        names = ['METADATA'] + tables
+        return max([len(x) for x in names]), len(str(len(names))) * 2 + 2
 
     def handle_run_start(self, run_result):
         self.progress.start()
@@ -78,18 +80,33 @@ class RichProfilerEventHandler(ProfilerEventHandler):
     def handle_run_end(self, run_result):
         self.progress.stop()
 
+    def handle_metadata_start(self):
+        self.progress.start()
+        padding = ' ' * (len(str(self.table_total)) - len(str(self.table_completed)))
+        coft = f'[{padding}{self.table_completed}/{self.table_total}]'
+        task_id = self.progress.add_task('METADATA', total=None, coft=coft)
+        self.tasks['__metadata__'] = task_id
+
+    def handle_metadata_progress(self, total, completed):
+        task_id = self.tasks['__metadata__']
+        self.progress.update(task_id, completed=completed, total=total)
+
+    def handle_metadata_end(self):
+        self.progress.stop()
+        task_id = self.tasks['__metadata__']
+        self.progress.remove_task(task_id)
+
     def handle_table_start(self, table_name):
+        self.progress.start()
         self.table_completed += 1
         padding = ' ' * (len(str(self.table_total)) - len(str(self.table_completed)))
         coft = f'[{padding}{self.table_completed}/{self.table_total}]'
         task_id = self.progress.add_task(table_name, total=None, coft=coft)
         self.tasks[table_name] = task_id
-        self.progress.start()
 
     def handle_table_progress(self, table_name, table_result, total, completed):
-        if completed == 0:
-            task_id = self.tasks[table_name]
-            self.progress.update(task_id, total=total)
+        task_id = self.tasks[table_name]
+        self.progress.update(task_id, total=total, completed=completed)
 
     def handle_table_end(self, table_name, table_result):
         self.progress.stop()
@@ -100,8 +117,7 @@ class RichProfilerEventHandler(ProfilerEventHandler):
         pass
 
     def handle_column_end(self, table_name, column_name, column_result):
-        task_id = self.tasks[table_name]
-        self.progress.update(task_id, advance=1)
+        pass
 
 
 class RichMetricEventHandler(MetricEventHandler):
@@ -159,6 +175,21 @@ class RichMetricEventHandler(MetricEventHandler):
         self.progress.update(task_id, advance=1)
 
 
+def _filter_subject(name: str, includes: List[str], excludes: List[str]) -> bool:
+    name = name.upper()
+    if includes is not None:
+        includes = [include.upper() for include in includes]
+        if name not in includes:
+            return False
+
+    if excludes is not None:
+        excludes = [exclude.upper() for exclude in excludes]
+        if name in excludes:
+            return False
+
+    return True
+
+
 def _execute_assertions(console: Console, engine, ds_name: str, output, profiler_result, created_at):
     # TODO: Implement running test cases based on profiling result
     assertion_engine = AssertionEngine(engine)
@@ -394,31 +425,6 @@ def _validate_assertions(console: Console):
     return False
 
 
-def _get_table_list(table, default_schema, dbt_adapter):
-    tables = None
-
-    if table:
-        table = re.sub(f'^({default_schema})\.', '', table)
-        tables = [table]
-
-    if dbt_adapter.is_ready():
-        dbt_tables = dbt_adapter.list_dbt_tables(default_schema)
-        if not dbt_tables:
-            return None, "No table found in dbt project."
-
-        if not table:
-            tables = dbt_tables
-        elif table not in dbt_tables:
-            suggestion = ''
-            lower_tables = [t.lower() for t in dbt_tables]
-            if table.lower() in lower_tables:
-                index = lower_tables.index(table.lower())
-                suggestion = f"Do you mean '{dbt_tables[index]}'?"
-            return None, f"Table '{table}' doesn't exist in dbt project. {suggestion}"
-
-    return tables, None
-
-
 def prepare_default_output_path(filesystem: FileSystem, created_at, ds):
     latest_symlink_path = os.path.join(filesystem.get_output_dir(), 'latest')
     latest_source = f"{ds.name}-{convert_to_tzlocal(created_at).strftime('%Y%m%d%H%M%S')}"
@@ -555,14 +561,41 @@ def _check_test_status(assertion_results, assertion_exceptions, dbt_test_results
     return True
 
 
+def check_dbt_manifest_compatibility(ds: DataSource, dbt_state_dir: str):
+    database = ds.get_database()
+    schema = ds.get_schema()
+
+    manifest = dbtutil._get_state_manifest(dbt_state_dir)
+    models = [x for x in manifest.get('nodes').values() if x.get('resource_type') in ['model']]
+
+    # all model’s schema should be the prefix of target’s schema
+    def filter_schema(model):
+        model_schema = model.get('schema', '')
+        return model_schema == schema or model_schema.startswith(f'{schema}_')
+    filtered_models = list(filter(filter_schema, models))
+
+    if len(models) != len(filtered_models):
+        return False
+
+    # at least one model uses the target’s database
+    return len([x for x in models if x.get('database') == database])
+
+
 class Runner():
     @staticmethod
     def exec(datasource=None, table=None, output=None, skip_report=False, dbt_state_dir: str = None,
-             report_dir: str = None):
+             dbt_run_results: bool = False, dbt_resources: Optional[dict] = None, report_dir: str = None):
         console = Console()
 
         raise_exception_when_directory_not_writable(output)
 
+        if table and dbt_run_results:
+            console.print("[bold red]Error:[/bold red] '--dbt-run-results' cannot be used with '--table'")
+            return sys.exit(1)
+        if dbt_resources and dbt_run_results:
+            console.print("[bold red]Error:[/bold red] Cannot specify dbt resources with '--dbt-run-results'")
+            return sys.exit(1)
+
         configuration = Configuration.load()
         filesystem = FileSystem(report_dir=report_dir)
         datasources = {}
@@ -575,13 +608,18 @@ class Runner():
             console.print("[bold red]Error: no datasource found[/bold red]")
             return 1
 
-        if datasource and datasource not in datasource_names:
-            console.print(f"[bold red]Error: datasource '{datasource}' doesn't exist[/bold red]")
+        if datasource:
+            ds_name = datasource
+        else:
+            # if global dbt config exists, use dbt profile target
+            # else use the first datasource
+            ds_name = configuration.dbt.get('target') if configuration.dbt else datasource_names[0]
+
+        if ds_name not in datasource_names:
+            console.print(f"[bold red]Error: datasource '{ds_name}' doesn't exist[/bold red]")
             console.print(f"Available datasources: {', '.join(datasource_names)}")
             return 1
 
-        # Use the first datasource if no datasource is specified
-        ds_name = datasource if datasource else datasource_names[0]
         ds = datasources[ds_name]
 
         passed, reasons = ds.validate()
@@ -593,7 +631,7 @@ class Runner():
                 "[bold yellow]Hint:[/bold yellow]\n  Please execute command 'piperider init' to move forward.")
             return 1
 
-        if not datasource and len(datasource_names) > 1:
+        if not datasource and configuration.dbt is None and len(datasource_names) > 1:
             console.print(
                 f"[bold yellow]Warning: multiple datasources found ({', '.join(datasource_names)}), using '{ds_name}'[/bold yellow]\n")
 
@@ -620,51 +658,76 @@ class Runner():
 
         if dbt_config and not dbtutil.is_ready(dbt_config):
             console.log('[bold red]ERROR:[/bold red] DBT configuration is not completed, please check the config.yml')
-            return 1
+            return sys.exit(1)
 
         console.rule('Profiling')
         run_id = uuid.uuid4().hex
         created_at = datetime.utcnow()
         engine = ds.get_engine_by_database()
-        default_schema = ds.credential.get('schema')
 
-        if default_schema is None and ds.type_name == 'bigquery':
-            default_schema = ds.credential.get('dataset')
-
-        if default_schema is None:
-            inspector = inspect(engine)
-            default_schema = inspector.default_schema_name
-
-        tables = None
+        subjects: List[ProfileSubject]
         dbt_test_results = None
-        if dbt_state_dir:
-            if not dbtutil.is_dbt_state_ready(dbt_state_dir):
-                console.print(
-                    f"[bold red]Error:[/bold red] No available 'manifest.json' or 'run_results.json' under '{dbt_state_dir}'")
-                return 1
-
-            tables = dbtutil.get_dbt_state_candidate(dbt_state_dir, configuration.include_views,
-                                                     configuration.includes, configuration.excludes)
-            dbt_test_results = dbtutil.get_dbt_state_tests_result(dbt_state_dir)
-
         if table:
             if len(table.split('.')) == 2:
                 schema, table_name = table.split('.')
-                tables = [ProfileSubject(table_name, schema, table_name)]
+                subjects = [ProfileSubject(table_name, schema)]
             else:
-                tables = [ProfileSubject(table, default_schema, table)]
-            # cli --table is specified, no inclusion and exclusion applied
-            configuration.includes = None
-            configuration.excludes = None
+                subjects = [ProfileSubject(table)]
+        else:
+            def filter_fn(subject: ProfileSubject):
+                return _filter_subject(subject.name, configuration.includes, configuration.excludes)
+
+            if dbt_config:
+                if not dbt_state_dir:
+                    dbt_project = dbtutil.load_dbt_project(dbt_config.get('projectDir'))
+                    dbt_state_dir = dbt_project.get('target-path')
+                if not dbtutil.is_dbt_state_ready(dbt_state_dir):
+                    console.print(
+                        f"[bold red]Error:[/bold red] No available 'manifest.json' under '{dbt_state_dir}'")
+                    return sys.exit(1)
+
+                if not check_dbt_manifest_compatibility(ds, dbt_state_dir):
+                    console.print("[bold red]Error:[/bold red] Target mismatched. "
+                                  f"Please run 'dbt compile -t {dbt_config.get('target')}' to generate the new manifest")
+                    return sys.exit(1)
+
+                if dbt_run_results:
+                    if not dbtutil.is_dbt_run_results_ready(dbt_state_dir):
+                        console.print(
+                            f"[bold red]Error:[/bold red] No available 'run_results.json' under '{dbt_state_dir}'")
+                        return sys.exit(1)
+                    dbt_test_results = dbtutil.get_dbt_state_tests_result(dbt_state_dir)
+
+                subjects = []
+                options = dict(
+                    view_profile=configuration.include_views,
+                    dbt_resources=dbt_resources,
+                    dbt_run_results=dbt_run_results,
+                    tag=dbt_config.get('tag')
+                )
+                candidate_nodes = dbtutil.get_dbt_state_candidate(dbt_state_dir, options)
+                for node in candidate_nodes:
+                    name = node.get('name')
+                    table = node.get('alias')
+                    schema = node.get('schema')
+                    database = node.get('database')
+                    subjects.append(ProfileSubject(table, schema, database, name))
+
+                if not dbt_resources:
+                    subjects = list(filter(filter_fn, subjects))
+            else:
+                table_names = inspect(engine).get_table_names()
+                if configuration.include_views:
+                    table_names += inspect(engine).get_view_names()
+
+                subjects = [ProfileSubject(table_name) for table_name in table_names]
+                subjects = list(filter(filter_fn, subjects))
 
         run_result = {}
-        available_tables = inspect(engine).get_table_names()
-        if configuration.include_views:
-            available_tables += inspect(engine).get_view_names()
-        available_tables = [t.table for t in tables] if tables else available_tables
-        profiler = Profiler(ds, RichProfilerEventHandler(available_tables), configuration)
+
+        profiler = Profiler(ds, RichProfilerEventHandler([subject.name for subject in subjects]), configuration)
         try:
-            profiler_result = profiler.profile(tables)
+            profiler_result = profiler.profile(subjects)
             run_result.update(profiler_result)
         except NoSuchTableError as e:
             console.print(f"[bold red]Error:[/bold red] No such table '{str(e)}'")
@@ -674,7 +737,7 @@ class Runner():
 
         metrics = []
         if dbt_state_dir:
-            metrics = dbtutil.get_dbt_state_metrics(dbt_state_dir)
+            metrics = dbtutil.get_dbt_state_metrics(dbt_state_dir, dbt_config.get('tag', 'piperider'), dbt_resources)
 
         if metrics:
             console.rule('Metrics')
